@@ -1,24 +1,14 @@
 package Method::Generate::Accessor;
-use strict;
-use warnings;
 
-use Moo::_Utils qw(_maybe_load_module _install_coderef _module_name_rx);
+use Moo::_strictures;
+use Moo::_Utils;
 use Moo::Object ();
-BEGIN { our @ISA = qw(Moo::Object) }
-use Sub::Quote qw(quote_sub quoted_from_sub quotify sanitize_identifier);
+our @ISA = qw(Moo::Object);
+use Sub::Quote qw(quote_sub quoted_from_sub quotify);
 use Scalar::Util 'blessed';
-use Carp qw(croak);
+use overload ();
+use Module::Runtime qw(use_module);
 BEGIN {
-  our @CARP_NOT = qw(
-    Moo::_Utils
-    Moo::Object
-    Moo::Role
-  );
-}
-BEGIN {
-  *_CAN_WEAKEN_READONLY = (
-    "$]" < 5.008_003 or $ENV{MOO_TEST_PRE_583}
-  ) ? sub(){0} : sub(){1};
   our $CAN_HAZ_XS =
     !$ENV{MOO_XS_DISABLE}
       &&
@@ -31,27 +21,20 @@ BEGIN {
     (eval { Class::XSAccessor->VERSION('1.17') })
   ;
 }
-BEGIN {
-  package
-    Method::Generate::Accessor::_Generated;
-  $Carp::Internal{+__PACKAGE__} = 1;
-}
 
-sub _die_overwrite {
+my $module_name_only = qr/\A$Module::Runtime::module_name_rx\z/;
+
+sub _die_overwrite
+{
   my ($pkg, $method, $type) = @_;
-  croak "You cannot overwrite a locally defined method ($method) with "
+  die "You cannot overwrite a locally defined method ($method) with "
     . ( $type || 'an accessor' );
 }
 
 sub generate_method {
   my ($self, $into, $name, $spec, $quote_opts) = @_;
-  $quote_opts = {
-    no_defer => 1,
-    package => 'Method::Generate::Accessor::_Generated',
-    %{ $quote_opts||{} },
-  };
   $spec->{allow_overwrite}++ if $name =~ s/^\+//;
-  croak "Must have an is" unless my $is = $spec->{is};
+  die "Must have an is" unless my $is = $spec->{is};
   if ($is eq 'ro') {
     $spec->{reader} = $name unless exists $spec->{reader};
   } elsif ($is eq 'rw') {
@@ -65,7 +48,7 @@ sub generate_method {
     $spec->{reader} = $name unless exists $spec->{reader};
     $spec->{writer} = "_set_${name}" unless exists $spec->{writer};
   } elsif ($is ne 'bare') {
-    croak "Unknown is ${is}";
+    die "Unknown is ${is}";
   }
   if (exists $spec->{builder}) {
     if(ref $spec->{builder}) {
@@ -75,8 +58,8 @@ sub generate_method {
       $spec->{builder} = 1;
     }
     $spec->{builder} = '_build_'.$name if ($spec->{builder}||0) eq 1;
-    croak "Invalid builder for $into->$name - not a valid method name"
-      if $spec->{builder} !~ _module_name_rx;
+    die "Invalid builder for $into->$name - not a valid method name"
+      if $spec->{builder} !~ $module_name_only;
   }
   if (($spec->{predicate}||0) eq 1) {
     $spec->{predicate} = $name =~ /^_/ ? "_has${name}" : "has_${name}";
@@ -94,7 +77,7 @@ sub generate_method {
     } elsif (blessed $isa and $isa->can('coerce')) {
       $spec->{coerce} = sub { $isa->coerce(@_) };
     } else {
-      croak "Invalid coercion for $into->$name - no appropriate type constraint";
+      die "Invalid coercion for $into->$name - no appropriate type constraint";
     }
   }
 
@@ -132,10 +115,9 @@ sub generate_method {
       $self->{captures} = {};
       $methods{$reader} =
         quote_sub "${into}::${reader}"
-          => '    Carp::croak("'.$reader.' is a read-only accessor") if @_ > 1;'."\n"
+          => '    die "'.$reader.' is a read-only accessor" if @_ > 1;'."\n"
              .$self->_generate_get($name, $spec)
           => delete $self->{captures}
-          => $quote_opts
         ;
     }
   }
@@ -156,7 +138,6 @@ sub generate_method {
         quote_sub "${into}::${accessor}"
           => $self->_generate_getset($name, $spec)
           => delete $self->{captures}
-          => $quote_opts
         ;
     }
   }
@@ -176,7 +157,6 @@ sub generate_method {
         quote_sub "${into}::${writer}"
           => $self->_generate_set($name, $spec)
           => delete $self->{captures}
-          => $quote_opts
         ;
     }
   }
@@ -188,27 +168,21 @@ sub generate_method {
         exists_predicates => $into, $pred, $name, $spec
       );
     } else {
-      $self->{captures} = {};
       $methods{$pred} =
-        quote_sub "${into}::${pred}"
-          => $self->_generate_simple_has('$_[0]', $name, $spec)."\n"
-          => delete $self->{captures}
-          => $quote_opts
+        quote_sub "${into}::${pred}" =>
+          '    '.$self->_generate_simple_has('$_[0]', $name, $spec)."\n"
         ;
     }
   }
-  if (my $builder = delete $spec->{builder_sub}) {
-    _install_coderef( "${into}::$spec->{builder}" => $builder );
+  if (my $pred = $spec->{builder_sub}) {
+    _install_coderef( "${into}::$spec->{builder}" => $spec->{builder_sub} );
   }
   if (my $cl = $spec->{clearer}) {
     _die_overwrite($into, $cl, 'a clearer')
       if !$spec->{allow_overwrite} && defined &{"${into}::${cl}"};
-    $self->{captures} = {};
     $methods{$cl} =
-      quote_sub "${into}::${cl}"
-        => $self->_generate_simple_clear('$_[0]', $name, $spec)."\n"
-        => delete $self->{captures}
-        => $quote_opts
+      quote_sub "${into}::${cl}" =>
+        $self->_generate_simple_clear('$_[0]', $name, $spec)."\n"
       ;
   }
   if (my $hspec = $spec->{handles}) {
@@ -220,10 +194,9 @@ sub generate_method {
         map [ $_ => ref($hspec->{$_}) ? @{$hspec->{$_}} : $hspec->{$_} ],
           keys %$hspec;
       } elsif (!ref($hspec)) {
-        require Moo::Role;
-        map [ $_ => $_ ], Moo::Role->methods_provided_by($hspec)
+        map [ $_ => $_ ], use_module('Moo::Role')->methods_provided_by(use_module($hspec))
       } else {
-        croak "You gave me a handles of ${hspec} and I have no idea why";
+        die "You gave me a handles of ${hspec} and I have no idea why";
       }
     };
     foreach my $delegation_spec (@specs) {
@@ -232,53 +205,24 @@ sub generate_method {
         if !$spec->{allow_overwrite} && defined &{"${into}::${proxy}"};
       $self->{captures} = {};
       $methods{$proxy} =
-        quote_sub "${into}::${proxy}"
-          => $self->_generate_delegation($asserter, $target, \@args)
-          => delete $self->{captures}
-          => $quote_opts
+        quote_sub "${into}::${proxy}" =>
+          $self->_generate_delegation($asserter, $target, \@args),
+          delete $self->{captures}
         ;
     }
   }
   if (my $asserter = $spec->{asserter}) {
-    _die_overwrite($into, $asserter, 'an asserter')
-      if !$spec->{allow_overwrite} && defined &{"${into}::${asserter}"};
-    local $self->{captures} = {};
+    $self->{captures} = {};
+
+
     $methods{$asserter} =
-      quote_sub "${into}::${asserter}"
-        => $self->_generate_asserter($name, $spec)
-        => delete $self->{captures}
-        => $quote_opts
-      ;
+      quote_sub "${into}::${asserter}" =>
+        $self->_generate_asserter($name, $spec),
+        delete $self->{captures};
   }
   \%methods;
 }
 
-sub merge_specs {
-  my ($self, @specs) = @_;
-  my $spec = shift @specs;
-  for my $old_spec (@specs) {
-    foreach my $key (keys %$old_spec) {
-      if ($key eq 'handles') {
-      }
-      elsif ($key eq 'moosify') {
-        $spec->{$key} = [
-          map { ref $_ eq 'ARRAY' ? @$_ : $_ }
-          grep defined,
-          ($old_spec->{$key}, $spec->{$key})
-        ];
-      }
-      elsif ($key eq 'builder' || $key eq 'default') {
-        $spec->{$key} = $old_spec->{$key}
-          if !(exists $spec->{builder} || exists $spec->{default});
-      }
-      elsif (!exists $spec->{$key}) {
-        $spec->{$key} = $old_spec->{$key};
-      }
-    }
-  }
-  $spec;
-}
-
 sub is_simple_attribute {
   my ($self, $name, $spec) = @_;
   # clearer doesn't have to be listed because it doesn't
@@ -398,34 +342,36 @@ sub _generate_simple_get {
 
 sub _generate_set {
   my ($self, $name, $spec) = @_;
-  my ($me, $source) = ('$_[0]', '$_[1]');
   if ($self->is_simple_set($name, $spec)) {
-    return $self->_generate_simple_set($me, $name, $spec, $source);
-  }
-
-  my ($coerce, $trigger, $isa_check) = @{$spec}{qw(coerce trigger isa)};
-  if ($coerce) {
-    $source = $self->_generate_coerce($name, $source, $coerce);
-  }
-  if ($isa_check) {
-    'scalar do { my $value = '.$source.";\n"
-    .'  ('.$self->_generate_isa_check($name, '$value', $isa_check)."),\n"
-    .'  ('.$self->_generate_simple_set($me, $name, $spec, '$value')."),\n"
-    .($trigger
-      ? '('.$self->_generate_trigger($name, $me, '$value', $trigger)."),\n"
-      : '')
-    .'  ('.$self->_generate_simple_get($me, $name, $spec)."),\n"
-    ."}";
-  }
-  elsif ($trigger) {
-    my $set = $self->_generate_simple_set($me, $name, $spec, $source);
-    "scalar (\n"
-    . '  ('.$self->_generate_trigger($name, $me, "($set)", $trigger)."),\n"
-    . '  ('.$self->_generate_simple_get($me, $name, $spec)."),\n"
-    . ")";
-  }
-  else {
-    '('.$self->_generate_simple_set($me, $name, $spec, $source).')';
+    $self->_generate_simple_set('$_[0]', $name, $spec, '$_[1]');
+  } else {
+    my ($coerce, $trigger, $isa_check) = @{$spec}{qw(coerce trigger isa)};
+    my $value_store = '$_[0]';
+    my $code;
+    if ($coerce) {
+      $value_store = '$value';
+      $code = "do { my (\$self, \$value) = \@_;\n"
+        ."        \$value = "
+        .$self->_generate_coerce($name, $value_store, $coerce).";\n";
+    }
+    else {
+      $code = "do { my \$self = shift;\n";
+    }
+    if ($isa_check) {
+      $code .=
+        "        ".$self->_generate_isa_check($name, $value_store, $isa_check).";\n";
+    }
+    my $simple = $self->_generate_simple_set('$self', $name, $spec, $value_store);
+    if ($trigger) {
+      my $fire = $self->_generate_trigger($name, '$self', $value_store, $trigger);
+      $code .=
+        "        ".$simple.";\n        ".$fire.";\n"
+        ."        $value_store;\n";
+    } else {
+      $code .= "        ".$simple.";\n";
+    }
+    $code .= "      }";
+    $code;
   }
 }
 
@@ -481,18 +427,24 @@ sub _wrap_attr_exception {
   .'    name     => '.quotify($name).",\n"
   .'    step     => '.quotify($step).",\n"
   ."  };\n"
-  .($want_return ? '  (my $_return),'."\n" : '')
-  .'  (my $_error), (my $_old_error = $@);'."\n"
-  ."  (eval {\n"
-  .'    ($@ = $_old_error),'."\n"
-  .'    ('
-  .($want_return ? '$_return ='."\n" : '')
-  .$code."),\n"
-  ."    1\n"
-  ."  } or\n"
-  .'    $_error = CORE::ref $@ ? $@ : '.$prefix.'.$@);'."\n"
-  .'  ($@ = $_old_error),'."\n"
-  .'  (defined $_error and CORE::die $_error);'."\n"
+  .($want_return ? '  my $_return;'."\n" : '')
+  .'  my $_error;'."\n"
+  ."  {\n"
+  .'    my $_old_error = $@;'."\n"
+  ."    if (!eval {\n"
+  .'      $@ = $_old_error;'."\n"
+  .($want_return ? '      $_return ='."\n" : '')
+  .'      '.$code.";\n"
+  ."      1;\n"
+  ."    }) {\n"
+  .'      $_error = $@;'."\n"
+  .'      if (!ref $_error) {'."\n"
+  .'        $_error = '.$prefix.'.$_error;'."\n"
+  ."      }\n"
+  ."    }\n"
+  .'    $@ = $_old_error;'."\n"
+  ."  }\n"
+  .'  die $_error if $_error;'."\n"
   .($want_return ? '  $_return;'."\n" : '')
   ."}\n"
 }
@@ -518,7 +470,7 @@ sub _generate_call_code {
     }
     my $code = $quoted->[1];
     if (my $captures = $quoted->[2]) {
-      my $cap_name = qq{\$${type}_captures_for_}.sanitize_identifier($name);
+      my $cap_name = qq{\$${type}_captures_for_}.$self->_sanitize_name($name);
       $self->{captures}->{$cap_name} = \$captures;
       Sub::Quote::inlinify($code, $values,
         Sub::Quote::capture_unroll($cap_name, $captures, 6), $local);
@@ -526,13 +478,17 @@ sub _generate_call_code {
       Sub::Quote::inlinify($code, $values, undef, $local);
     }
   } else {
-    my $cap_name = qq{\$${type}_for_}.sanitize_identifier($name);
+    my $cap_name = qq{\$${type}_for_}.$self->_sanitize_name($name);
     $self->{captures}->{$cap_name} = \$sub;
     "${cap_name}->(${values})";
   }
 }
 
-sub _sanitize_name { sanitize_identifier($_[1]) }
+sub _sanitize_name {
+  my ($self, $name) = @_;
+  $name =~ s/([_\W])/sprintf('_%x', ord($1))/ge;
+  $name;
+}
 
 sub generate_populate_set {
   my $self = shift;
@@ -543,45 +499,68 @@ sub generate_populate_set {
 
 sub _generate_populate_set {
   my ($self, $me, $name, $spec, $source, $test, $init_arg) = @_;
-
-  my $has_default = $self->has_eager_default($name, $spec);
-  if (!($has_default || $test)) {
-    return '';
-  }
-  if ($has_default) {
-    my $get_default = $self->_generate_get_default($me, $name, $spec);
-    $source =
-      $test
-        ? "(\n  ${test}\n"
-            ."   ? ${source}\n   : "
+  if ($self->has_eager_default($name, $spec)) {
+    my $get_indent = ' ' x ($spec->{isa} ? 6 : 4);
+    my $get_default = $self->_generate_get_default(
+                        '$new', $name, $spec
+                      );
+    my $get_value =
+      defined($spec->{init_arg})
+        ? "(\n${get_indent}  ${test}\n"
+            ."${get_indent}   ? ${source}\n${get_indent}   : "
             .$get_default
-            .")"
+            ."\n${get_indent})"
         : $get_default;
-  }
-  if ($spec->{coerce}) {
-    $source = $self->_generate_coerce(
-      $name, $source,
-      $spec->{coerce}, $init_arg
+    if ($spec->{coerce}) {
+      $get_value = $self->_generate_coerce(
+        $name, $get_value,
+        $spec->{coerce}, $init_arg
+      )
+    }
+    ($spec->{isa}
+      ? "    {\n      my \$value = ".$get_value.";\n      "
+        .$self->_generate_isa_check(
+          $name, '$value', $spec->{isa}, $init_arg
+        ).";\n"
+        .'      '.$self->_generate_simple_set($me, $name, $spec, '$value').";\n"
+        ."    }\n"
+      : '    '.$self->_generate_simple_set($me, $name, $spec, $get_value).";\n"
     )
-  }
-  if ($spec->{isa}) {
-    $source = 'scalar do { my $value = '.$source.";\n"
-    .'  ('.$self->_generate_isa_check(
-        $name, '$value', $spec->{isa}, $init_arg
-      )."),\n"
-    ."  \$value\n"
-    ."}\n";
-  }
-  my $set = $self->_generate_simple_set($me, $name, $spec, $source);
-  my $trigger = $spec->{trigger} ? $self->_generate_trigger(
-    $name, $me, $self->_generate_simple_get($me, $name, $spec),
-    $spec->{trigger}
-  ) : undef;
-  if ($has_default) {
-    "($set)," . ($trigger && $test ? "($test and $trigger)," : '') . "\n";
-  }
-  else {
-    "($test and ($set)" . ($trigger ? ", ($trigger)" : '') . "),\n";
+    .($spec->{trigger}
+      ? '    '
+        .$self->_generate_trigger(
+          $name, $me, $self->_generate_simple_get($me, $name, $spec),
+          $spec->{trigger}
+        )." if ${test};\n"
+      : ''
+    );
+  } else {
+    "    if (${test}) {\n"
+      .($spec->{coerce}
+        ? "      $source = "
+          .$self->_generate_coerce(
+            $name, $source,
+            $spec->{coerce}, $init_arg
+          ).";\n"
+        : ""
+      )
+      .($spec->{isa}
+        ? "      "
+          .$self->_generate_isa_check(
+            $name, $source, $spec->{isa}, $init_arg
+          ).";\n"
+        : ""
+      )
+      ."      ".$self->_generate_simple_set($me, $name, $spec, $source).";\n"
+      .($spec->{trigger}
+        ? "      "
+          .$self->_generate_trigger(
+            $name, $me, $self->_generate_simple_get($me, $name, $spec),
+            $spec->{trigger}
+          ).";\n"
+        : ""
+      )
+      ."    }\n";
   }
 }
 
@@ -609,23 +588,21 @@ sub _generate_simple_set {
     #
     # but requires Internal functions and is just too damn crazy
     # so simply throw a better exception
-    my $weak_simple = _CAN_WEAKEN_READONLY
-      ? "do { Scalar::Util::weaken(${simple}); no warnings 'void'; $get }"
-      : <<"EOC"
-        ( eval { Scalar::Util::weaken($simple); 1 }
-          ? do { no warnings 'void'; $get }
-          : do {
-            if( \$@ =~ /Modification of a read-only value attempted/) {
-              require Carp;
-              Carp::croak( sprintf (
-                'Reference to readonly value in "%s" can not be weakened on Perl < 5.8.3',
-                $name_str,
-              ) );
-            } else {
-              die \$@;
-            }
+    my $weak_simple = "do { Scalar::Util::weaken(${simple}); no warnings 'void'; $get }";
+    Moo::_Utils::lt_5_8_3() ? <<"EOC" : $weak_simple;
+      eval { Scalar::Util::weaken($simple); 1 }
+        ? do { no warnings 'void'; $get }
+        : do {
+          if( \$@ =~ /Modification of a read-only value attempted/) {
+            require Carp;
+            Carp::croak( sprintf (
+              'Reference to readonly value in "%s" can not be weakened on Perl < 5.8.3',
+              $name_str,
+            ) );
+          } else {
+            die \$@;
           }
-        )
+        }
 EOC
   } else {
     $simple;
@@ -640,11 +617,12 @@ sub _generate_getset {
 
 sub _generate_asserter {
   my ($self, $name, $spec) = @_;
-  my $name_str = quotify($name);
+
   "do {\n"
    ."  my \$val = ".$self->_generate_get($name, $spec).";\n"
-   ."  ".$self->_generate_simple_has('$_[0]', $name, $spec)."\n"
-   ."    or Carp::croak(q{Attempted to access '}.${name_str}.q{' but it is not set});\n"
+   ."  unless (".$self->_generate_simple_has('$_[0]', $name, $spec).") {\n"
+   .qq!    die "Attempted to access '${name}' but it is not set";\n!
+   ."  }\n"
    ."  \$val;\n"
    ."}\n";
 }
@@ -677,27 +655,19 @@ sub default_construction_string { '{}' }
 
 sub _validate_codulatable {
   my ($self, $setting, $value, $into, $appended) = @_;
+  my $invalid = "Invalid $setting '" . overload::StrVal($value)
+    . "' for $into not a coderef";
+  $invalid .= " $appended" if $appended;
 
-  my $error;
-
-  if (blessed $value) {
-    local $@;
-    no warnings 'void';
-    eval { \&$value; 1 }
-      and return 1;
-    $error = "could not be converted to a coderef: $@";
-  }
-  elsif (ref $value eq 'CODE') {
-    return 1;
-  }
-  else {
-    $error = 'is not a coderef or code-convertible object';
+  unless (ref $value and (ref $value eq 'CODE' or blessed($value))) {
+    die "$invalid or code-convertible object";
   }
 
-  croak "Invalid $setting '"
-    . ($INC{'overload.pm'} ? overload::StrVal($value) : $value)
-    . "' for $into " . $error
-    . ($appended ? " $appended" : '');
+  unless (eval { \&$value }) {
+    die "$invalid and could not be converted to a coderef: $@";
+  }
+
+  1;
 }
 
 1;
