@@ -1,39 +1,40 @@
 package Method::Generate::Constructor;
-use strict;
-use warnings;
 
-use Sub::Quote qw(quote_sub quotify);
+use Moo::_strictures;
+use Sub::Quote qw(quote_sub unquote_sub quotify);
 use Sub::Defer;
-use Moo::_Utils qw(_getstash _getglob _linear_isa);
-use Scalar::Util qw(weaken);
-use Carp qw(croak);
-use Carp::Heavy ();
-BEGIN { our @CARP_NOT = qw(Sub::Defer) }
-BEGIN {
-  local $Moo::sification::disabled = 1;
-  require Moo;
-  Moo->import;
-}
+use Moo::_Utils qw(_getstash _getglob);
+use Moo;
 
 sub register_attribute_specs {
   my ($self, @new_specs) = @_;
   $self->assert_constructor;
   my $specs = $self->{attribute_specs}||={};
-  my $ag = $self->accessor_generator;
   while (my ($name, $new_spec) = splice @new_specs, 0, 2) {
     if ($name =~ s/^\+//) {
-      croak "has '+${name}' given but no ${name} attribute already exists"
+      die "has '+${name}' given but no ${name} attribute already exists"
         unless my $old_spec = $specs->{$name};
-      $ag->merge_specs($new_spec, $old_spec);
+      foreach my $key (keys %$old_spec) {
+        if (!exists $new_spec->{$key}) {
+          $new_spec->{$key} = $old_spec->{$key}
+            unless $key eq 'handles';
+        }
+        elsif ($key eq 'moosify') {
+          $new_spec->{$key} = [
+            map { ref $_ eq 'ARRAY' ? @$_ : $_ }
+              ($old_spec->{$key}, $new_spec->{$key})
+          ];
+        }
+      }
     }
     if ($new_spec->{required}
       && !(
-        $ag->has_default($name, $new_spec)
+        $self->accessor_generator->has_default($name, $new_spec)
         || !exists $new_spec->{init_arg}
         || defined $new_spec->{init_arg}
       )
     ) {
-      croak "You cannot have a required attribute (${name})"
+      die "You cannot have a required attribute (${name})"
         . " without a default, builder, or an init_arg";
     }
     $new_spec->{index} = scalar keys %$specs
@@ -75,32 +76,25 @@ sub install_delayed {
   my ($self) = @_;
   $self->assert_constructor;
   my $package = $self->{package};
-  my (undef, @isa) = @{_linear_isa($package)};
+  my (undef, @isa) = @{mro::get_linear_isa($package)};
   my $isa = join ',', @isa;
-  my (undef, $from_file, $from_line) = caller(Carp::short_error_loc());
-  my $constructor = defer_sub "${package}::new" => sub {
-    my (undef, @new_isa) = @{_linear_isa($package)};
+  $self->{deferred_constructor} = defer_sub "${package}::new" => sub {
+    my (undef, @new_isa) = @{mro::get_linear_isa($package)};
     if (join(',', @new_isa) ne $isa) {
       my ($expected_new) = grep { *{_getglob($_.'::new')}{CODE} } @isa;
       my ($found_new) = grep { *{_getglob($_.'::new')}{CODE} } @new_isa;
       if (($found_new||'') ne ($expected_new||'')) {
         $found_new ||= 'none';
         $expected_new ||= 'none';
-        croak "Expected parent constructor of $package to be"
+        die "Expected parent constructor of $package expected to be"
         . " $expected_new, but found $found_new: changing the inheritance"
-        . " chain (\@ISA) at runtime (after $from_file line $from_line) is unsupported";
+        . " chain (\@ISA) at runtime is unsupported";
       }
     }
-
-    my $constructor = $self->generate_method(
-      $package, 'new', $self->{attribute_specs}, { no_install => 1, no_defer => 1 }
-    );
-    $self->{inlined} = 1;
-    weaken($self->{constructor} = $constructor);
-    $constructor;
+    unquote_sub $self->generate_method(
+      $package, 'new', $self->{attribute_specs}, { no_install => 1 }
+    )
   };
-  $self->{inlined} = 0;
-  weaken($self->{constructor} = $constructor);
   $self;
 }
 
@@ -114,44 +108,41 @@ sub assert_constructor {
   my $package = $self->{package} or return 1;
   my $current = $self->current_constructor($package)
     or return 1;
-  my $constructor = $self->{constructor}
-    or croak "Unknown constructor for $package already exists";
-  croak "Constructor for $package has been replaced with an unknown sub"
-    if $constructor != $current;
-  croak "Constructor for $package has been inlined and cannot be updated"
-    if $self->{inlined};
+  my $deferred = $self->{deferred_constructor}
+    or die "Unknown constructor for $package already exists";
+  return 1
+    if $deferred == $current;
+  my $current_deferred = (Sub::Defer::defer_info($current)||[])->[3];
+  if ($current_deferred && $current_deferred == $deferred) {
+    die "Constructor for $package has been inlined and cannot be updated";
+  }
+  die "Constructor for $package has been replaced with an unknown sub";
 }
 
 sub generate_method {
   my ($self, $into, $name, $spec, $quote_opts) = @_;
-  $quote_opts = {
-    %{$quote_opts||{}},
-    package => $into,
-  };
   foreach my $no_init (grep !exists($spec->{$_}{init_arg}), keys %$spec) {
     $spec->{$no_init}{init_arg} = $no_init;
   }
   local $self->{captures} = {};
-
+  my $body = '    my $class = shift;'."\n"
+            .'    $class = ref($class) if ref($class);'."\n";
+  $body .= $self->_handle_subconstructor($into, $name);
   my $into_buildargs = $into->can('BUILDARGS');
-
-  my $body
-    = '    my $invoker = CORE::shift();'."\n"
-    . '    my $class = CORE::ref($invoker) ? CORE::ref($invoker) : $invoker;'."\n"
-    . $self->_handle_subconstructor($into, $name)
-    . ( $into_buildargs && $into_buildargs != \&Moo::Object::BUILDARGS
-      ? $self->_generate_args_via_buildargs
-      : $self->_generate_args
-    )
-    . $self->_check_required($spec)
-    . '    my $new = '.$self->construction_string.";\n"
-    . $self->_assign_new($spec)
-    . ( $into->can('BUILD')
-      ? $self->buildall_generator->buildall_body_for( $into, '$new', '$args' )
-      : ''
-    )
-    . '    return $new;'."\n";
-
+  if ( $into_buildargs && $into_buildargs != \&Moo::Object::BUILDARGS ) {
+      $body .= $self->_generate_args_via_buildargs;
+  } else {
+      $body .= $self->_generate_args;
+  }
+  $body .= $self->_check_required($spec);
+  $body .= '    my $new = '.$self->construction_string.";\n";
+  $body .= $self->_assign_new($spec);
+  if ($into->can('BUILD')) {
+    $body .= $self->buildall_generator->buildall_body_for(
+      $into, '$new', '$args'
+    );
+  }
+  $body .= '    return $new;'."\n";
   if ($into->can('DEMOLISH')) {
     require Method::Generate::DemolishAll;
     Method::Generate::DemolishAll->new->generate_method($into);
@@ -182,7 +173,7 @@ sub _cap_call {
 sub _generate_args_via_buildargs {
   my ($self) = @_;
   q{    my $args = $class->BUILDARGS(@_);}."\n"
-  .q{    Carp::croak("BUILDARGS did not return a hashref") unless CORE::ref($args) eq 'HASH';}
+  .q{    die "BUILDARGS did not return a hashref" unless ref($args) eq 'HASH';}
   ."\n";
 }
 
@@ -190,16 +181,21 @@ sub _generate_args_via_buildargs {
 sub _generate_args {
   my ($self) = @_;
   return <<'_EOA';
-    my $args = scalar @_ == 1
-      ? CORE::ref $_[0] eq 'HASH'
-        ? { %{ $_[0] } }
-        : Carp::croak("Single parameters to new() must be a HASH ref"
-            . " data => ". $_[0])
-      : @_ % 2
-        ? Carp::croak("The new() method for $class expects a hash reference or a"
-            . " key/value list. You passed an odd number of arguments")
-        : {@_}
-    ;
+    my $args;
+    if ( scalar @_ == 1 ) {
+        unless ( defined $_[0] && ref $_[0] eq 'HASH' ) {
+            die "Single parameters to new() must be a HASH ref"
+                ." data => ". $_[0] ."\n";
+        }
+        $args = { %{ $_[0] } };
+    }
+    elsif ( @_ % 2 ) {
+        die "The new() method for $class expects a hash reference or a"
+          . " key/value list. You passed an odd number of arguments\n";
+    }
+    else {
+        $args = {@_};
+    }
 _EOA
 
 }
@@ -215,13 +211,12 @@ sub _assign_new {
     $test{$name} = $attr_spec->{init_arg};
   }
   join '', map {
-    my $arg = $test{$_};
-    my $arg_key = quotify($arg);
-    my $test = defined $arg ? "exists \$args->{$arg_key}" : undef;
-    my $source = defined $arg ? "\$args->{$arg_key}" : undef;
+    my $arg_key = quotify($test{$_});
+    my $test = "exists \$args->{$arg_key}";
+    my $source = "\$args->{$arg_key}";
     my $attr_spec = $spec->{$_};
     $self->_cap_call($ag->generate_populate_set(
-      '$new', $_, $attr_spec, $source, $test, $arg,
+      '$new', $_, $attr_spec, $source, $test, $test{$_},
     ));
   } sort keys %test;
 }
@@ -231,13 +226,13 @@ sub _check_required {
   my @required_init =
     map $spec->{$_}{init_arg},
       grep {
-        my $s = $spec->{$_}; # ignore required if default or builder set
-        $s->{required} and not($s->{builder} or exists $s->{default})
+        my %s = %{$spec->{$_}}; # ignore required if default or builder set
+        $s{required} and not($s{builder} or exists $s{default})
       } sort keys %$spec;
   return '' unless @required_init;
   '    if (my @missing = grep !exists $args->{$_}, '
     .join(', ', map quotify($_), @required_init).') {'."\n"
-    .q{      Carp::croak("Missing required arguments: ".CORE::join(', ', sort @missing));}."\n"
+    .q{      die "Missing required arguments: ".join(', ', sort @missing);}."\n"
     ."    }\n";
 }
 
@@ -259,8 +254,5 @@ Moo->_constructor_maker_for(__PACKAGE__)
   subconstructor_handler => { is => 'ro' },
   package => { is => 'bare' },
 );
-if ($INC{'Moo/HandleMoose.pm'} && !$Moo::sification::disabled) {
-  Moo::HandleMoose::inject_fake_metaclass_for(__PACKAGE__);
-}
 
 1;
