@@ -2,15 +2,9 @@ package Method::Generate::DemolishAll;
 
 use Moo::_strictures;
 use Moo::Object ();
-BEGIN { our @ISA = qw(Moo::Object) }
+our @ISA = qw(Moo::Object);
 use Sub::Quote qw(quote_sub quotify);
-use Moo::_Utils qw(_getglob);
-use Moo::_mro;
-BEGIN {
-  *_USE_DGD = "$]" < 5.014 ? sub(){1} : sub(){0};
-  require Devel::GlobalDestruction
-    if _USE_DGD();
-}
+use Moo::_Utils;
 
 sub generate_method {
   my ($self, $into) = @_;
@@ -23,15 +17,10 @@ sub generate_method {
     q!    my $self = shift;
     my $e = do {
       local $?;
-      local $@;!.(_USE_DGD ? q!
-      require Devel::GlobalDestruction;! : '').q!
-      package !.$into.q!;
+      local $@;
+      require Devel::GlobalDestruction;
       eval {
-        $self->DEMOLISHALL(!.(
-          _USE_DGD
-            ? 'Devel::GlobalDestruction::in_global_destruction()'
-            : q[${^GLOBAL_PHASE} eq 'DESTRUCT']
-        ).q!);
+        $self->DEMOLISHALL(Devel::GlobalDestruction::in_global_destruction);
       };
       $@;
     };
@@ -49,15 +38,12 @@ sub demolishall_body_for {
     grep *{_getglob($_)}{CODE},
     map "${_}::DEMOLISH",
     @{mro::get_linear_isa($into)};
-  join '',
-    qq{    package $into;\n},
-    map qq{    ${me}->${_}(${args});\n}, @demolishers;
+  join '', map qq{    ${me}->${_}(${args});\n}, @demolishers;
 }
 
 sub _handle_subdemolish {
   my ($self, $into) = @_;
   '    if (ref($_[0]) ne '.quotify($into).') {'."\n".
-  "      package $into;\n".
   '      return shift->Moo::Object::DEMOLISHALL(@_)'.";\n".
   '    }'."\n";
 }
