@@ -1,8 +1,6 @@
 package Moo::Object;
-use strict;
-use warnings;
 
-use Carp ();
+use Moo::_strictures;
 
 our %NO_BUILD;
 our %NO_DEMOLISH;
@@ -30,17 +28,21 @@ sub new {
 
 # Inlined into Method::Generate::Constructor::_generate_args() - keep in sync
 sub BUILDARGS {
-  my $class = shift;
-  scalar @_ == 1
-    ? ref $_[0] eq 'HASH'
-      ? { %{ $_[0] } }
-      : Carp::croak("Single parameters to new() must be a HASH ref"
-          . " data => ". $_[0])
-    : @_ % 2
-      ? Carp::croak("The new() method for $class expects a hash reference or a"
-          . " key/value list. You passed an odd number of arguments")
-      : {@_}
-  ;
+    my $class = shift;
+    if ( scalar @_ == 1 ) {
+        unless ( defined $_[0] && ref $_[0] eq 'HASH' ) {
+            die "Single parameters to new() must be a HASH ref"
+                ." data => ". $_[0] ."\n";
+        }
+        return { %{ $_[0] } };
+    }
+    elsif ( @_ % 2 ) {
+        die "The new() method for $class expects a hash reference or a"
+          . " key/value list. You passed an odd number of arguments\n";
+    }
+    else {
+        return {@_};
+    }
 }
 
 sub BUILDALL {
