@@ -5,7 +5,7 @@ use Exporter qw(import);
 use Moo::_Utils qw(_getglob _install_coderef);
 use Scalar::Util qw(weaken);
 
-our $VERSION = '2.000001';
+our $VERSION = '2.000002';
 $VERSION = eval $VERSION;
 
 our @EXPORT = qw(defer_sub undefer_sub undefer_all);
@@ -72,7 +72,7 @@ sub defer_sub {
 
 sub CLONE {
   %DEFERRED = map { defined $_ && $_->[3] ? ($_->[3] => $_) : () } values %DEFERRED;
-  for my $info (values %DEFERRED) {
+  foreach my $info (values %DEFERRED) {
     weaken($info)
       unless $info->[0] && ${$info->[2]};
   }
